package secret

import (
	"encoding/json"
	"fmt"
	"log/slog"
	"os"
	"path/filepath"
	"strings"
	"time"

	"filippo.io/age"
	"git.eeqj.de/sneak/secret/pkg/agehd"
	"github.com/spf13/afero"
)

// VaultInterface defines the interface that vault implementations must satisfy
type VaultInterface interface {
	GetDirectory() (string, error)
	AddSecret(name string, value []byte, force bool) error
	GetName() string
	GetFilesystem() afero.Fs
	GetCurrentUnlockKey() (UnlockKey, error)
	CreatePassphraseKey(passphrase string) (*PassphraseUnlockKey, error)
}

// Secret represents a secret in a vault
type Secret struct {
	Name      string
	Directory string
	Metadata  SecretMetadata
	vault     VaultInterface
}

// NewSecret creates a new Secret instance
func NewSecret(vault VaultInterface, name string) *Secret {
	DebugWith("Creating new secret instance",
		slog.String("secret_name", name),
		slog.String("vault_name", vault.GetName()),
	)

	// Convert slashes to percent signs for storage directory name
	storageName := strings.ReplaceAll(name, "/", "%")
	vaultDir, _ := vault.GetDirectory()
	secretDir := filepath.Join(vaultDir, "secrets.d", storageName)

	DebugWith("Secret storage details",
		slog.String("secret_name", name),
		slog.String("storage_name", storageName),
		slog.String("secret_dir", secretDir),
	)

	return &Secret{
		Name:      name,
		Directory: secretDir,
		vault:     vault,
		Metadata: SecretMetadata{
			Name:      name,
			CreatedAt: time.Now(),
			UpdatedAt: time.Now(),
		},
	}
}

// Save saves a secret value to the vault
func (s *Secret) Save(value []byte, force bool) error {
	DebugWith("Saving secret",
		slog.String("secret_name", s.Name),
		slog.String("vault_name", s.vault.GetName()),
		slog.Int("value_length", len(value)),
		slog.Bool("force", force),
	)

	err := s.vault.AddSecret(s.Name, value, force)
	if err != nil {
		Debug("Failed to save secret", "error", err, "secret_name", s.Name)
		return err
	}

	Debug("Successfully saved secret", "secret_name", s.Name)
	return nil
}

// GetValue retrieves and decrypts the secret value using the provided unlock key
func (s *Secret) GetValue(unlockKey UnlockKey) ([]byte, error) {
	DebugWith("Getting secret value",
		slog.String("secret_name", s.Name),
		slog.String("vault_name", s.vault.GetName()),
	)

	// Check if secret exists
	exists, err := s.Exists()
	if err != nil {
		Debug("Failed to check if secret exists during GetValue", "error", err, "secret_name", s.Name)
		return nil, fmt.Errorf("failed to check if secret exists: %w", err)
	}
	if !exists {
		Debug("Secret not found during GetValue", "secret_name", s.Name, "vault_name", s.vault.GetName())
		return nil, fmt.Errorf("secret %s not found", s.Name)
	}

	Debug("Secret exists, proceeding with decryption", "secret_name", s.Name)

	// Check if we have SB_SECRET_MNEMONIC environment variable for direct decryption
	if envMnemonic := os.Getenv(EnvMnemonic); envMnemonic != "" {
		Debug("Using mnemonic from environment for direct long-term key derivation", "secret_name", s.Name)

		// Use mnemonic directly to derive long-term key
		ltIdentity, err := agehd.DeriveIdentity(envMnemonic, 0)
		if err != nil {
			Debug("Failed to derive long-term key from mnemonic for secret", "error", err, "secret_name", s.Name)
			return nil, fmt.Errorf("failed to derive long-term key from mnemonic: %w", err)
		}

		Debug("Successfully derived long-term key from mnemonic", "secret_name", s.Name)

		// Use the long-term key to decrypt the secret using per-secret architecture
		return s.decryptWithLongTermKey(ltIdentity)
	}

	Debug("Using unlock key for vault access", "secret_name", s.Name)

	// Use the provided unlock key to get the vault's long-term private key
	if unlockKey == nil {
		Debug("No unlock key provided for secret decryption", "secret_name", s.Name)
		return nil, fmt.Errorf("unlock key required to decrypt secret")
	}

	DebugWith("Getting vault's long-term key using unlock key",
		slog.String("secret_name", s.Name),
		slog.String("unlock_key_type", unlockKey.GetType()),
		slog.String("unlock_key_id", unlockKey.GetID()),
	)

	// Step 1: Use the unlock key to get the vault's long-term private key
	unlockIdentity, err := unlockKey.GetIdentity()
	if err != nil {
		Debug("Failed to get unlock key identity", "error", err, "secret_name", s.Name, "unlock_key_type", unlockKey.GetType())
		return nil, fmt.Errorf("failed to get unlock key identity: %w", err)
	}

	// Read the encrypted long-term private key from the unlock key directory
	encryptedLtPrivKeyPath := filepath.Join(unlockKey.GetDirectory(), "longterm.age")
	Debug("Reading encrypted long-term private key", "path", encryptedLtPrivKeyPath)

	encryptedLtPrivKey, err := afero.ReadFile(s.vault.GetFilesystem(), encryptedLtPrivKeyPath)
	if err != nil {
		Debug("Failed to read encrypted long-term private key", "error", err, "path", encryptedLtPrivKeyPath)
		return nil, fmt.Errorf("failed to read encrypted long-term private key: %w", err)
	}

	// Decrypt the encrypted long-term private key using the unlock key
	Debug("Decrypting long-term private key using unlock key", "secret_name", s.Name)
	ltPrivKeyData, err := DecryptWithIdentity(encryptedLtPrivKey, unlockIdentity)
	if err != nil {
		Debug("Failed to decrypt long-term private key", "error", err, "secret_name", s.Name)
		return nil, fmt.Errorf("failed to decrypt long-term private key: %w", err)
	}

	// Parse the long-term private key
	Debug("Parsing long-term private key", "secret_name", s.Name)
	ltIdentity, err := age.ParseX25519Identity(string(ltPrivKeyData))
	if err != nil {
		Debug("Failed to parse long-term private key", "error", err, "secret_name", s.Name)
		return nil, fmt.Errorf("failed to parse long-term private key: %w", err)
	}

	DebugWith("Successfully obtained vault's long-term key",
		slog.String("secret_name", s.Name),
		slog.String("public_key", ltIdentity.Recipient().String()),
	)

	// Use the long-term key to decrypt the secret using per-secret architecture
	return s.decryptWithLongTermKey(ltIdentity)
}

// decryptWithLongTermKey decrypts the secret using the vault's long-term private key
// This implements the per-secret key architecture: longterm -> secret private key -> secret value
func (s *Secret) decryptWithLongTermKey(ltIdentity *age.X25519Identity) ([]byte, error) {
	DebugWith("Decrypting secret with long-term key using per-secret architecture",
		slog.String("secret_name", s.Name),
		slog.String("vault_name", s.vault.GetName()),
	)

	// Step 1: Read the secret's encrypted private key from priv.age
	encryptedSecretPrivKeyPath := filepath.Join(s.Directory, "priv.age")
	Debug("Reading encrypted secret private key", "path", encryptedSecretPrivKeyPath)

	encryptedSecretPrivKey, err := afero.ReadFile(s.vault.GetFilesystem(), encryptedSecretPrivKeyPath)
	if err != nil {
		Debug("Failed to read encrypted secret private key", "error", err, "path", encryptedSecretPrivKeyPath)
		return nil, fmt.Errorf("failed to read encrypted secret private key: %w", err)
	}

	DebugWith("Read encrypted secret private key",
		slog.String("secret_name", s.Name),
		slog.Int("encrypted_length", len(encryptedSecretPrivKey)),
	)

	// Step 2: Decrypt the secret's private key using the vault's long-term private key
	Debug("Decrypting secret private key using long-term key", "secret_name", s.Name)
	secretPrivKeyData, err := DecryptWithIdentity(encryptedSecretPrivKey, ltIdentity)
	if err != nil {
		Debug("Failed to decrypt secret private key", "error", err, "secret_name", s.Name)
		return nil, fmt.Errorf("failed to decrypt secret private key: %w", err)
	}

	// Parse the secret's private key
	Debug("Parsing secret's private key", "secret_name", s.Name)
	secretIdentity, err := age.ParseX25519Identity(string(secretPrivKeyData))
	if err != nil {
		Debug("Failed to parse secret's private key", "error", err, "secret_name", s.Name)
		return nil, fmt.Errorf("failed to parse secret's private key: %w", err)
	}

	DebugWith("Successfully decrypted and parsed secret's identity",
		slog.String("secret_name", s.Name),
		slog.String("secret_public_key", secretIdentity.Recipient().String()),
	)

	// Step 3: Read the secret's encrypted value from value.age
	encryptedValuePath := filepath.Join(s.Directory, "value.age")
	Debug("Reading encrypted secret value", "path", encryptedValuePath)

	encryptedValue, err := afero.ReadFile(s.vault.GetFilesystem(), encryptedValuePath)
	if err != nil {
		Debug("Failed to read encrypted secret value", "error", err, "path", encryptedValuePath)
		return nil, fmt.Errorf("failed to read encrypted secret value: %w", err)
	}

	DebugWith("Read encrypted secret value",
		slog.String("secret_name", s.Name),
		slog.Int("encrypted_length", len(encryptedValue)),
	)

	// Step 4: Decrypt the secret's value using the secret's private key
	Debug("Decrypting value using secret key", "secret_name", s.Name)
	decryptedValue, err := DecryptWithIdentity(encryptedValue, secretIdentity)
	if err != nil {
		Debug("Failed to decrypt secret value", "error", err, "secret_name", s.Name)
		return nil, fmt.Errorf("failed to decrypt secret value: %w", err)
	}

	DebugWith("Successfully decrypted secret value using per-secret key architecture",
		slog.String("secret_name", s.Name),
		slog.Int("decrypted_length", len(decryptedValue)),
	)

	return decryptedValue, nil
}

// LoadMetadata loads the secret metadata from disk
func (s *Secret) LoadMetadata() error {
	DebugWith("Loading secret metadata",
		slog.String("secret_name", s.Name),
		slog.String("vault_name", s.vault.GetName()),
	)

	vaultDir, err := s.vault.GetDirectory()
	if err != nil {
		Debug("Failed to get vault directory for metadata loading", "error", err, "secret_name", s.Name)
		return err
	}

	// Convert slashes to percent signs for storage
	storageName := strings.ReplaceAll(s.Name, "/", "%")
	metadataPath := filepath.Join(vaultDir, "secrets.d", storageName, "secret-metadata.json")

	DebugWith("Reading secret metadata",
		slog.String("secret_name", s.Name),
		slog.String("metadata_path", metadataPath),
	)

	// Read metadata file
	metadataBytes, err := afero.ReadFile(s.vault.GetFilesystem(), metadataPath)
	if err != nil {
		Debug("Failed to read secret metadata file", "error", err, "metadata_path", metadataPath)
		return fmt.Errorf("failed to read metadata: %w", err)
	}

	DebugWith("Read secret metadata file",
		slog.String("secret_name", s.Name),
		slog.Int("metadata_size", len(metadataBytes)),
	)

	var metadata SecretMetadata
	if err := json.Unmarshal(metadataBytes, &metadata); err != nil {
		Debug("Failed to parse secret metadata JSON", "error", err, "secret_name", s.Name)
		return fmt.Errorf("failed to parse metadata: %w", err)
	}

	DebugWith("Parsed secret metadata",
		slog.String("secret_name", metadata.Name),
		slog.Time("created_at", metadata.CreatedAt),
		slog.Time("updated_at", metadata.UpdatedAt),
	)

	s.Metadata = metadata
	Debug("Successfully loaded secret metadata", "secret_name", s.Name)
	return nil
}

// GetMetadata returns the secret metadata
func (s *Secret) GetMetadata() SecretMetadata {
	Debug("Returning secret metadata", "secret_name", s.Name)
	return s.Metadata
}

// GetEncryptedData reads and returns the encrypted secret data
func (s *Secret) GetEncryptedData() ([]byte, error) {
	DebugWith("Getting encrypted secret data",
		slog.String("secret_name", s.Name),
		slog.String("vault_name", s.vault.GetName()),
	)

	secretPath := filepath.Join(s.Directory, "value.age")

	Debug("Reading encrypted secret file", "secret_path", secretPath)

	encryptedData, err := afero.ReadFile(s.vault.GetFilesystem(), secretPath)
	if err != nil {
		Debug("Failed to read encrypted secret file", "error", err, "secret_path", secretPath)
		return nil, fmt.Errorf("failed to read encrypted secret: %w", err)
	}

	DebugWith("Successfully read encrypted secret data",
		slog.String("secret_name", s.Name),
		slog.Int("encrypted_length", len(encryptedData)),
	)

	return encryptedData, nil
}

// Exists checks if the secret exists on disk
func (s *Secret) Exists() (bool, error) {
	DebugWith("Checking if secret exists",
		slog.String("secret_name", s.Name),
		slog.String("vault_name", s.vault.GetName()),
	)

	secretPath := filepath.Join(s.Directory, "value.age")

	Debug("Checking secret file existence", "secret_path", secretPath)

	exists, err := afero.Exists(s.vault.GetFilesystem(), secretPath)
	if err != nil {
		Debug("Failed to check secret file existence", "error", err, "secret_path", secretPath)
		return false, err
	}

	DebugWith("Secret existence check result",
		slog.String("secret_name", s.Name),
		slog.Bool("exists", exists),
	)

	return exists, nil
}

// GetCurrentVault gets the current vault from the file system
// This function is a wrapper around the actual implementation in the vault package
// and exists to break the import cycle.
func GetCurrentVault(fs afero.Fs, stateDir string) (VaultInterface, error) {
	// This is a forward declaration. The actual implementation is provided
	// by the vault package when it calls RegisterGetCurrentVaultFunc.
	if getCurrentVaultFunc == nil {
		return nil, fmt.Errorf("GetCurrentVault function not registered")
	}
	return getCurrentVaultFunc(fs, stateDir)
}

// getCurrentVaultFunc is a function variable that will be set by the vault package
// to implement the actual GetCurrentVault functionality
var getCurrentVaultFunc func(fs afero.Fs, stateDir string) (VaultInterface, error)

// RegisterGetCurrentVaultFunc allows the vault package to register its implementation
// of GetCurrentVault to break the import cycle
func RegisterGetCurrentVaultFunc(fn func(fs afero.Fs, stateDir string) (VaultInterface, error)) {
	getCurrentVaultFunc = fn
}
